/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.logging.Logger;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import org.apache.commons.io.IOUtils;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactException;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ConfigurationPropertiesCallback;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.artifact.lifecycle.ArtifactLifecycle;
import org.ow2.petals.admin.api.artifact.lifecycle.ArtifactLifecycleFactory;
import org.ow2.petals.admin.api.artifact.lifecycle.ComponentLifecycle;
import org.ow2.petals.admin.api.conf.ArtifactUrlRewriter;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactStoppedException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.admin.api.exception.ArtifactVersionIsNeededException;
import org.ow2.petals.admin.api.exception.ComponentRequiredException;
import org.ow2.petals.admin.api.exception.UnsupportedArtifactTypeException;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Identification;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;
import org.ow2.petals.jbi.descriptor.original.generated.Target;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public abstract class ArtifactAdministration {

    private static final Logger LOG = Logger.getLogger(ArtifactAdministration.class.getName());

    private static final String TARGET_SU_ZIP_ARCHIVE = "service-unit.zip";

    protected final ArtifactLifecycleFactory lifecycleFactory;

    protected ArtifactAdministration(final PetalsAdministration admin, final ArtifactUrlRewriter artifactUrlRewriter) {
        this.lifecycleFactory = admin.newArtifactLifecycleFactory(artifactUrlRewriter);
    }

    private final void deployAndStart(final Artifact artifact, final URL artifactUrl, final boolean skipStartup)
            throws ArtifactAdministrationException {
        final Artifact artifactToDeploy;
        final URL artifactToDeployUrl;
        File tempFileCreated = null;

        try {
            if (artifact instanceof ServiceUnit) {
                tempFileCreated = File.createTempFile("sa-" + artifact.getName() + "-", ".zip");
                artifactToDeploy = this.createSatoDeploySu((ServiceUnit) artifact, tempFileCreated, artifactUrl);
                artifactToDeployUrl = tempFileCreated.toURI().toURL();

            } else {
                artifactToDeploy = artifact;
                artifactToDeployUrl = artifactUrl;
            }

            final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory.createLifecycle(artifactToDeploy);
            artifactLifecycle.deploy(artifactToDeployUrl);
            if (!skipStartup) {
                artifactLifecycle.start();
            }
        } catch (final IOException e) {
            throw new ArtifactAdministrationException(e);
        } finally {
            if (tempFileCreated != null) {
                tempFileCreated.delete();
            }
        }
    }

    /**
     * Create a {@link ServiceAssembly} and its archive to deploy the deployable service unit provided.
     * 
     * @param artifact
     *            The service unit
     * @param saFile
     *            The archive of the resulting service assembly
     * @param suUrl
     *            The archive of the provided service unit
     * @return The resulting {@link ServiceAssembly}
     * @throws ArtifactAdministrationException
     */
    private final ServiceAssembly createSatoDeploySu(final ServiceUnit serviceUnit, final File saFile, final URL suUrl)
            throws ArtifactAdministrationException {
        
        final ServiceAssembly serviceAssembly = new ServiceAssembly(serviceUnit.getName());
        serviceAssembly.addServiceUnit(serviceUnit);
        
        final Jbi saJbiDescr = new Jbi();
        saJbiDescr.setVersion(BigDecimal.valueOf(1));
        final org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly saJbiDescrSa = new org.ow2.petals.jbi.descriptor.original.generated.ServiceAssembly();
        saJbiDescr.setServiceAssembly(saJbiDescrSa);
        final Identification saJbiDescrIdent = new Identification();
        saJbiDescrSa.setIdentification(saJbiDescrIdent);
        saJbiDescrIdent.setName(serviceUnit.getName());
        saJbiDescrIdent.setDescription("SA automatically generated to deploy SU '" + serviceUnit.getName() + "'");
        final org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit saJbiDescrSu = new org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit();
        saJbiDescrSa.getServiceUnit().add(saJbiDescrSu);
        final Identification saJbiDescrSuIdent = new Identification();
        saJbiDescrSuIdent.setName(serviceUnit.getName());
        saJbiDescrSuIdent.setDescription("Original SU deployed");
        saJbiDescrSu.setIdentification(saJbiDescrSuIdent);
        final Target saJbiDescrSuTarget = new Target();
        saJbiDescrSuTarget.setArtifactsZip(TARGET_SU_ZIP_ARCHIVE);
        saJbiDescrSuTarget.setComponentName(serviceUnit.getTargetComponent());
        saJbiDescrSu.setTarget(saJbiDescrSuTarget);
        
        try {
            final ZipOutputStream saZos = new ZipOutputStream(new FileOutputStream(saFile));
            try {
                final ZipEntry jbiEntry = new ZipEntry(JBIDescriptorBuilder.JBI_DESCRIPTOR_RESOURCE);
                saZos.putNextEntry(jbiEntry);
                try {
                    JBIDescriptorBuilder.getInstance().writeXMLJBIdescriptor(saJbiDescr, saZos);
                } finally {
                    saZos.closeEntry();
                }
                final ZipEntry suArchiveEntry = new ZipEntry(TARGET_SU_ZIP_ARCHIVE);
                saZos.putNextEntry(suArchiveEntry);
                try {
                    final InputStream isServiceUnit = suUrl.openStream();
                    try {
                        IOUtils.copy(isServiceUnit, saZos);
                    } finally {
                        IOUtils.closeQuietly(isServiceUnit);
                    }
                } finally {
                    saZos.closeEntry();
                }
            } catch (final JBIDescriptorException | IOException e) {
                throw new ArtifactAdministrationException(e);
            } finally {
                IOUtils.closeQuietly(saZos);
            }

            return serviceAssembly;

        } catch (final FileNotFoundException e) {
            throw new ArtifactAdministrationException(e);
        }

    }

    /**
     * Deploy and start the artifact given by its URL, taking into account the artifact type.
     * 
     * @param artifactUrl
     *            Url of the artifact to deploy and eventually start
     * @param skipStartup
     *            if <code>true</code> the artifact will be only deployed, it will not be started.
     * @throws ArtifactAdministrationException
     */
    public void deployAndStartArtifact(final URL artifactUrl, final boolean skipStartup)
            throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
            this.deployAndStart(artifact, artifactUrl, skipStartup);
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * Deploy and start the artifact given by its URL, taking into account the artifact type.
     * 
     * @param artifactUrl
     *            Url of the artifact to deploy and eventually start
     * @param configurationProperties
     *            Parameters of the artifact are given through these properties
     * @param skipStartup
     *            if <code>true</code> the artifact will be only deployed, it will not be started.
     * @throws ArtifactAdministrationException
     */
    public void deployAndStartArtifact(final URL artifactUrl, final Properties configurationProperties,
            final boolean skipStartup)
            throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl, configurationProperties);
            this.deployAndStart(artifact, artifactUrl, skipStartup);
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }

    }

    /**
     * Deploy and start the artifact given by its URL, taking into account the artifact type.
     * 
     * @param artifactUrl
     *            Url of the artifact to deploy and eventually start
     * @param configurationPropertiesCallback
     *            Callback used to compute configuration parameter values from old ones
     * @param skipStartup
     *            if <code>true</code> the artifact will be only deployed, it will not be started.
     * @throws ArtifactAdministrationException
     */
    public void deployAndStartArtifact(final URL artifactUrl,
            final ConfigurationPropertiesCallback configurationPropertiesCallback, final boolean skipStartup)
            throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl, configurationPropertiesCallback);
            this.deployAndStart(artifact, artifactUrl, skipStartup);
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }

    }

    /**
     * Deploy and start the artifact given by its URL, taking into account the artifact type.
     * 
     * @param artifactUrl
     *            Url of the artifact to deploy and eventually start
     * @param configurationFile
     *            Parameters of the artifact are given through the property file located at this URL
     * @param skipStartup
     *            if <code>true</code> the artifact will be only deployed, it will not be started.
     * @throws ArtifactAdministrationException
     */
    public void deployAndStartArtifact(final URL artifactUrl, final URL configurationFile, final boolean skipStartup)
            throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl, configurationFile);
            this.deployAndStart(artifact, artifactUrl, skipStartup);
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * Start an artifact.
     * 
     * @param artifactUrl
     *            {@link URL of the artifact to start}
     * 
     * @throws ArtifactStartedException
     *             The artifact is already started
     * @throws ArtifactNotDeployedException
     *             The artifact is not deployed
     * @throws ArtifactNotFoundException
     *             The artifact can not be found
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact start.
     */
    public void startArtifact(final URL artifactUrl) throws ArtifactStartedException,
            ArtifactNotDeployedException, ArtifactNotFoundException,
            ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
            this.startArtifact(artifact);
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * <p>
     * Start an artifact.
     * </p>
     * <p>
     * The artifact type is needed to distinct between two JBI artifacts having the same identifier.
     * </p>
     * 
     * @param type
     *            The nature (SL, SE, BC, SA) of the artifact to start. Can be <code>null</code>.
     * 
     * @param name
     *            The JBI identifier of the artifact to start.
     * @throws UnsupportedArtifactTypeException
     *             <code>type</code> is a not supported artifact type.
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ArtifactTypeIsNeededException
     *             Several artifacts match the name, the artifact type MUST be provided to be able to find the right
     *             artifact.
     * @throws ArtifactStartedException
     *             The artifact is already started
     * @throws ArtifactNotDeployedException
     *             The artifact is not deployed
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact start.
     */
    public void startArtifact(final String type, final String name)
            throws UnsupportedArtifactTypeException, ArtifactNotFoundException,
            ArtifactTypeIsNeededException, ArtifactStartedException, ArtifactNotDeployedException,
            ArtifactAdministrationException {
        final Artifact artifact = this.getArtifact(type, name, null);
        this.startArtifact(artifact);
    }

    /**
     * Start an artifact.
     * 
     * @param artifact
     *            The artifact to start
     * 
     * @throws ArtifactStartedException
     *             The artifact is already started
     * @throws ArtifactNotDeployedException
     *             The artifact is not deployed
     * @throws ArtifactNotFoundException
     *             The artifact can not be found
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact start.
     */
    private void startArtifact(final Artifact artifact) throws ArtifactStartedException,
            ArtifactNotDeployedException, ArtifactNotFoundException,
            ArtifactAdministrationException {

        assert artifact != null : "artifact is null";

        final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory
                .createLifecycle(artifact);
        artifactLifecycle.start();
    }

    /**
     * Stop artifact
     * 
     * @param artifactUrl
     * @throws ArtifactAdministrationException
     */
    public void stopArtifact(final URL artifactUrl) throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
            ArtifactLifecycle artifactLifecycle = this.lifecycleFactory.createLifecycle(artifact);
            artifactLifecycle.stop();
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * <p>Stop an artifact.</p>
	 * <p>The artifact type is needed to distinct betwwen two JBI artifacts having the same
     * identifier.</p>
     * 
     * @param type
     *            The nature (SL, SA, BC, SA) of the artifact to stop.
     * 
     * @param name
     *            The JBI identifier of the artifact to stop.
     * @throws UnsupportedArtifactTypeException
     *             <code>type</code> is a not supported artifact type.
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ArtifactTypeIsNeededException
     *             Several artifacts match the name, the artifact type MUST be
     *             provided to be able to find the right artifact.
     * @throws ArtifactAdministrationException
     *             An error occurs retrieving the artifact.
     */
    public void stopArtifact(final String type, final String name) throws UnsupportedArtifactTypeException,
            ArtifactNotFoundException, ArtifactTypeIsNeededException,
            ArtifactAdministrationException {

        final Artifact artifact = this.getArtifact(type, name, null);
        final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory
                .createLifecycle(artifact);
        artifactLifecycle.stop();
    }

    public void stopAndUndeployArtifact(final URL artifactUrl) throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
            this.stopAndUndeployArtifact(artifact);
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * <p>
     * Stop and undeploy an artifact.
     * </p>
     * <p>
     * The artifact type is needed to distinct between two JBI artifacts having the same identifier.
     * </p>
     * <p>
     * The artifact version is needed to distinct between two versions of a shared library.
     * </p>
     * 
     * @param type
     *            artifact type (SL,BC,SE,SA). Can be <code>null</code>.
     * @param name
     *            Artifact identifier. Not <code>null</code>.
     * @param version
     *            Artifact version, for artifact supporting version. Can be <code>null</code>.
     * @throws UnsupportedArtifactTypeException
     *             <code>type</code> is a not supported artifact type.
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ArtifactTypeIsNeededException
     *             Several artifacts match the name, the artifact type MUST be provided to be able to find the right
     *             artifact.
     * @throws ArtifactAdministrationException
     *             An error occurs retrieving the artifact.
     */
    public void stopAndUndeployArtifact(final String type, final String name, final String version)
            throws UnsupportedArtifactTypeException, ArtifactNotFoundException,
            ArtifactTypeIsNeededException, ArtifactAdministrationException {
        final Artifact artifact = this.getArtifact(type, name, version);
        this.stopAndUndeployArtifact(artifact);
    }

    private void stopAndUndeployArtifact(final Artifact artifact) throws ArtifactAdministrationException {

        final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory
                .createLifecycle(artifact);
        try {
            artifactLifecycle.stop();
        } catch (final ArtifactStoppedException e) {
            // Artifact already stopped, we can continue the undeployment
            // lifecycle
        } catch (final ArtifactNotDeployedException e) {
            // Artifact not completely deployed, we can continue the
            // undeployment
        }

        artifactLifecycle.undeploy();
    }

    public Artifact getArtifactInfo(final URL artifactUrl) throws ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
            final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory
                    .createLifecycle(artifact);
            artifactLifecycle.updateState();
            return artifact;
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * <p>
     * Get information about an artifact already deployed.
     * </p>
     * <p>
     * The artifact type is needed to distinct between two JBI artifacts having the same identifier.
     * </p>
     * <p>
     * The artifact version is needed to distinct between two versions of a shared library.
     * </p>
     * 
     * @param type
     *            artifact type (SL,BC,SE,SA). Can be <code>null</code>.
     * @param name
     *            Artifact identifier. Not <code>null</code>.
     * @param version
     *            Artifact version, for artifact supporting version. Can be <code>null</code>.
     * @throws UnsupportedArtifactTypeException
     *             <code>type</code> is a not supported artifact type.
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ArtifactTypeIsNeededException
     *             Several artifacts match the name, the artifact type MUST be provided to be able to find the right
     *             artifact.
     * @throws ArtifactVersionIsNeededException
     *             Several artifacts match the name, the artifact version MUST be provided to be able to find the right
     *             artifact.
     * @throws ArtifactAdministrationException
     *             An error occurs retrieving the artifact.
     */
    public Artifact getArtifactInfo(final String type, final String name, final String version)
            throws UnsupportedArtifactTypeException, ArtifactNotFoundException, ArtifactTypeIsNeededException,
            ArtifactVersionIsNeededException, ArtifactAdministrationException {

        final Artifact artifact = this.getArtifact(type, name, version);
        final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory
                .createLifecycle(artifact);
        artifactLifecycle.updateState();
        return artifact;
    }

    /**
     * Deploy and start all specified artifacts taking into account the artifact
     * type.
     * 
     * @param artifactsUrl
     *            artifacts to deploy and start
     * @throws ArtifactAdministrationException
     */
    public void deployAndStartArtifacts(final URL[] artifactsUrl) throws ArtifactAdministrationException {
        this.deployAndStartArtifacts(artifactsUrl, false);
    }

    /**
     * Deploy and start all specified artifacts taking into account the artifact type.
     * 
     * @param artifactsUrl
     *            artifacts to deploy and eventually start
     * @param skipStartup
     *            if <code>true</code> the artifact will be only deployed, it will not be started.
     * @throws ArtifactAdministrationException
     */
    public void deployAndStartArtifacts(final URL[] artifactsUrl, final boolean skipStartup)
            throws ArtifactAdministrationException {

        final Map<URI, Artifact> sls = new HashMap<>();
        final Map<URI, Artifact> components = new HashMap<>();
        final Map<URI, Artifact> sas = new HashMap<>();
        final Map<URI, Artifact> unknows = new HashMap<>();

        try {
            for (final URL url : artifactsUrl) {
                try {
                    final Artifact artifact = ArtifactUtils.createArtifact(url);
                    if (artifact instanceof SharedLibrary) {
                        sls.put(url.toURI(), artifact);
                    } else if (artifact instanceof Component) {
                        components.put(url.toURI(), artifact);
                    } else if (artifact instanceof ServiceAssembly) {
                        sas.put(url.toURI(), artifact);
                    } else {
                        unknows.put(url.toURI(), artifact);
                    }
                } catch (final ArtifactException e) {
                    throw new ArtifactAdministrationException(e);
                }
            }

            final Map<URI, Artifact> orderedMap = new LinkedHashMap<>();
            orderedMap.putAll(sls);
            orderedMap.putAll(components);
            orderedMap.putAll(sas);
            orderedMap.putAll(unknows);

            for (final Entry<URI, Artifact> entry : orderedMap.entrySet()) {
                this.deployAndStart(entry.getValue(), entry.getKey().toURL(), skipStartup);
            }
        } catch (final MalformedURLException e) {
            throw new ArtifactAdministrationException(e);
        } catch (final URISyntaxException e) {
            throw new ArtifactAdministrationException(e);
        }
    }

    /**
     * <p>
     * Stop and undeploy all artifacts currently deployed in the container.
     * </p>
     * 
     * @throws ArtifactAdministrationException
     */
    public abstract void stopAndUndeployAllArtifacts() throws ArtifactAdministrationException;

    /**
     * <p>
     * Stop and undeploy all specified artifacts, taking into account the
     * artifact type.
     * </p>
     * <p>
     * If <code>artifactsUrl</code> is <code>null</code>, all artifacts
     * installed or deployed on the container will be stopped and undeployed.
     * </p>
     * <p>
     * If <code>artifactsUrl</code> is empty, no artifact will be stopped and
     * undeployed.
     * </p>
     * 
     * @param artifactsUrl
     *            artifacts to stop and undeploy. Can be <code>null</code> or
     *            empty.
     * @throws ArtifactAdministrationException
     */
    public void stopAndUndeployAllArtifacts(final URL[] artifactsUrl)
            throws ArtifactAdministrationException {
        if (artifactsUrl == null) {
            this.stopAndUndeployAllArtifacts();
        } else {
            try {
                // First, we undeploy service-assemblies
                for (final URL artifactUrl : artifactsUrl) {
                    final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
                    if (ServiceAssembly.TYPE.equals(artifact.getType())) {
                        try {
                            this.stopAndUndeployArtifact(artifactUrl);
                        } catch (final ArtifactNotFoundException e) {
                            // Artifact not deployed, we continue with a warning
                            LOG.warning(String
                                    .format("The artifact [%s] (type; %s) cannot be found. Undeployment skipped !",
                                            e.getName(), e.getType()));
                        }
                    }
                }
                // 2nd, we uninstall components
                for (final URL artifactUrl : artifactsUrl) {
                    final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
                    if (Component.ComponentType.BC.toString().equals(artifact.getType())
                            || Component.ComponentType.SE.toString().equals(artifact.getType())) {
                        try {
                            this.stopAndUndeployArtifact(artifactUrl);
                        } catch (final ArtifactNotFoundException e) {
                            // Artifact not deployed, we continue with a warning
                            LOG.warning(String
                                    .format("The artifact [%s] (type; %s) cannot be found. Undeployment skipped !",
                                            e.getName(), e.getType()));
                        }
                    }
                }
                // And we finish with shared libraries
                for (final URL artifactUrl : artifactsUrl) {
                    final Artifact artifact = ArtifactUtils.createArtifact(artifactUrl);
                    if (SharedLibrary.TYPE.equals(artifact.getType())) {
                        try {
                            this.stopAndUndeployArtifact(artifactUrl);
                        } catch (final ArtifactNotFoundException e) {
                            // Artifact not deployed, we continue with a warning
                            LOG.warning(String
                                    .format("The artifact [%s] (type; %s) cannot be found. Undeployment skipped !",
                                            e.getName(), e.getType()));
                        }
                    }
                }
            } catch (final ArtifactException e) {
                throw new ArtifactAdministrationException(e);
            }
        }
    }

    /**
     * <p>
     * Get the list of artifact deployed on the current Petals node.
     * </p>
     * <p>
     * Artifacts returned are:
     * <ul>
     * <li>shared libraries,</li>
     * <li>components, included components that have only their component
     * installer loaded,</li>
     * <li>service assemblies.</li>
     * </ul>
     * </p>
     * 
     * @return The list of artifact deployed on the current Petals node
     * @throws ArtifactAdministrationException
     */
    public abstract List<Artifact> listArtifacts() throws ArtifactAdministrationException;

    /**
     * <p>
     * Create an artifact with this type and name. The artifact to return is resolved by the underlying implementation.
     * </p>
     * <p>
     * The artifact type is needed to distinct betwwen two JBI artifacts having the same identifier.
     * </p>
     * <p>
     * The artifact version is needed to distinct between two versions of a shared library.
     * </p>
     * 
     * @param type
     *            artifact type (SL, BC, SE, SA). Can be <code>null</code>.
     * @param name
     *            Artifact identifier. Not <code>null</code>.
     * @param version
     *            Artifact version, for artifact supporting version. Can be <code>null</code>.
     * @return
     * @throws UnsupportedArtifactTypeException
     *             <code>type</code> is a not supported artifact type.
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ArtifactTypeIsNeededException
     *             Several artifacts match the name, the artifact type MUST be provided to be able to find the right
     *             artifact.
     * @throws ArtifactVersionIsNeededException
     *             Several artifacts match the name, the artifact version MUST be provided to be able to find the right
     *             artifact.
     * @throws ArtifactAdministrationException
     *             An error occurs retrieving the artifact.
     */
    public abstract Artifact getArtifact(final String type, final String name, final String version)
            throws UnsupportedArtifactTypeException, ArtifactNotFoundException, ArtifactTypeIsNeededException,
            ArtifactVersionIsNeededException, ArtifactAdministrationException;

    /**
     * Reload placeholders configuration of a CDK-based JBI component
     * 
     * @param componentUrl
     *            The URL of the component
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ComponentRequiredException
     *             The given artifact is not a JBI component.
     * @throws ArtifactAdministrationException
     *             An error occurs reloading the configuration of the component.
     */
    public void reloadConfiguration(final URL componentUrl) throws ArtifactNotFoundException,
            ComponentRequiredException,
            ArtifactAdministrationException {
        try {
            final Artifact artifact = ArtifactUtils.createArtifact(componentUrl);
            final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory.createLifecycle(artifact);
            if (artifactLifecycle instanceof ComponentLifecycle) {
                ((ComponentLifecycle) artifactLifecycle).reloadConfiguration();
            } else {
                throw new ComponentRequiredException(artifact.getName());
            }
        } catch (final ArtifactException ae) {
            throw new ArtifactAdministrationException(ae);
        }
    }

    /**
     * Reload placeholders configuration of a CDK-based JBI component.
     * 
     * @param name
     *            The JBI identifier of the component
     * @throws ArtifactNotFoundException
     *             No artifact found.
     * @throws ArtifactAdministrationException
     *             An error occurs retrieving the artifact.
     */
    public void reloadConfiguration(final String name) throws UnsupportedArtifactTypeException,
            ArtifactNotFoundException, ArtifactAdministrationException {

        Artifact artifact;
        try {
            artifact = this.getArtifact("BC", name, null);
        } catch (final ArtifactNotFoundException e) {
            artifact = this.getArtifact("SE", name, null);
        }

        final ArtifactLifecycle artifactLifecycle = this.lifecycleFactory.createLifecycle(artifact);
        if (artifactLifecycle instanceof ComponentLifecycle) {
            ((ComponentLifecycle) artifactLifecycle).reloadConfiguration();
        } else {
            // This case should not occur because only components should be returned by 'getArtifact("BC|SE", ...)'
            throw new ComponentRequiredException(artifact.getName());
        }
    }
}
