/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api.artifact;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import org.apache.commons.io.IOUtils;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.exception.ArtifactNameMissingInJBIDescriptorException;
import org.ow2.petals.admin.api.artifact.exception.ArtifactSlVersionMissingInJBIDescriptorException;
import org.ow2.petals.admin.api.artifact.exception.HttpCodeNotOkException;
import org.ow2.petals.admin.api.artifact.exception.HttpRemoteArtifactException;
import org.ow2.petals.admin.api.artifact.exception.InvalidContentTypeException;
import org.ow2.petals.admin.api.artifact.exception.InvalidZipArchiveException;
import org.ow2.petals.admin.api.artifact.exception.JBIDescriptorMissingException;
import org.ow2.petals.admin.api.artifact.exception.MissingContentTypeException;
import org.ow2.petals.admin.api.artifact.exception.SeveralServicesException;
import org.ow2.petals.admin.api.artifact.exception.TargetComponentMissingException;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.extension.JBIDescriptorExtensionBuilder;
import org.ow2.petals.jbi.descriptor.extension.exception.NoComponentNameDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.NoIdentificationDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.NotDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.extension.exception.SeveralIdentificationDeployableServiceUnitException;
import org.ow2.petals.jbi.descriptor.original.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.original.generated.Identification;
import org.ow2.petals.jbi.descriptor.original.generated.Jbi;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public final class ArtifactUtils {

    /**
     * Property key to set {@link ArtifactUtils} implementation name
     */
    public static final String FACTORY_PROPERTY_KEY = "org.ow2.petals.admin.api.artifact.factory";

    private ArtifactUtils() {
        // NOP: It's an utility class
    }

    /**
     * Create artifact from the specified URL
     * 
     * @param artifactUrl
     * @return instance of artifact
     * @throws ArtifactException
     */
    public static final Artifact createArtifact(final URL artifactUrl) throws ArtifactException {
        assert artifactUrl != null;
        return createArtifact(artifactUrl, new Properties());
    }

    /**
     * Create artifact from the specified URL with the specified properties
     * files (Only useful for {@link Component})
     * 
     * @param artifactUrl
     * @param configurationFile
     * @return The artifact associated to the URL
     * @throws ArtifactException
     *             An error occurs reading the JBI archive or the properties
     *             file
     */
    public static final Artifact createArtifact(URL artifactUrl, URL configurationFile)
            throws ArtifactException {
        assert artifactUrl != null;
        assert configurationFile != null;

        try {
            final Properties configurationProperties = new Properties();
            final File tmpConfigurationFile = downloadPropertiesFileToTempZipFile(configurationFile);
            try (final Reader fileReader = new FileReader(tmpConfigurationFile)) {
                configurationProperties.load(fileReader);
            }
            return createArtifact(artifactUrl, configurationProperties);
        } catch (final IOException e) {
            throw new ArtifactException(e);
        }
    }

    /**
     * <p>
     * Create artifact from the specified URL with the specified properties files (Only useful for {@link Component})
     * </p>
     * <p>
     * Configuration parameters are given through {@link Properties}.
     * </p>
     * 
     * @param artifactUrl
     * @param configurationProperties
     *            Configuration parameters given as {@link Properties}. Not <code>null</code>.
     * @return
     * @throws ArtifactException
     */
    public static final Artifact createArtifact(final URL artifactUrl, final Properties configurationProperties)
            throws ArtifactException {
        assert configurationProperties != null;

        return createArtifact(artifactUrl, null, configurationProperties);
    }

    /**
     * <p>
     * Create artifact from the specified URL with the specified properties files (Only useful for {@link Component}).
     * </p>
     * <p>
     * Configuration parameters are given through a callback.
     * </p>
     * 
     * @param artifactUrl
     * @param configurationPropertiesCallback
     *            A call back to compute configuration parameters from the one already configured in the component. Not
     *            <code>null</code>.
     * @return
     * @throws ArtifactException
     */
    public static final Artifact createArtifact(final URL artifactUrl,
            final ConfigurationPropertiesCallback configurationPropertiesCallback)
            throws ArtifactException {
        assert configurationPropertiesCallback != null;

        return createArtifact(artifactUrl, configurationPropertiesCallback, null);
    }

    /**
     * <p>
     * Create artifact from the specified URL with the specified properties files (Only useful for {@link Component}).
     * </p>
     * <p>
     * Configuration parameters are given by <code>configurationPropertiesCallback</code> or
     * <code>configurationProperties</code>, not both in the same time.
     * </p>
     * 
     * @param artifactUrl
     * @param configurationPropertiesCallback
     *            A call back to compute configuration parameters from the one already configured in the component. Can
     *            be <code>null</code> if <code>configurationProperties</code> is not <code>null</code>.
     * @param configurationProperties
     *            Configuration parameters given as {@link Properties}. Can be <code>null</code> if
     *            <code>configurationPropertiesCallback</code> is not <code>null</code>.
     * @return
     * @throws ArtifactException
     */
    private static final Artifact createArtifact(final URL artifactUrl,
            final ConfigurationPropertiesCallback configurationPropertiesCallback,
            final Properties configurationProperties) throws ArtifactException {
        assert artifactUrl != null;
        assert configurationPropertiesCallback == null ^ configurationProperties == null;

        try {
            final File tmpJbiFile = downloadArtifactToTempZipFile(artifactUrl);
            final Jbi jbiConfig;
            try {
                jbiConfig = JBIDescriptorBuilder.getInstance().buildJavaJBIDescriptorFromArchive(tmpJbiFile);
            } catch (JBIDescriptorException e) {
                if (e.getMessage().contains("JBI descriptor not found")) {
                    throw new JBIDescriptorMissingException(artifactUrl);
                } else {
                    throw new InvalidZipArchiveException("Invalid artifact", artifactUrl, e);
                }
            } finally {
                tmpJbiFile.delete();
            }

            if (jbiConfig.getComponent() != null) {
                ComponentType type = jbiConfig.getComponent()
                        .getType() == org.ow2.petals.jbi.descriptor.original.generated.ComponentType.BINDING_COMPONENT
                                ? ComponentType.BC : ComponentType.SE;
                String name = getName(jbiConfig.getComponent().getIdentification(), artifactUrl);
                Set<SharedLibrary> sls = getSLs(jbiConfig.getComponent());
                if (configurationPropertiesCallback != null) {
                    return new Component(name, type, sls, configurationPropertiesCallback);
                } else {
                    return new Component(name, type, sls, configurationProperties);
                }
            } else if (jbiConfig.getSharedLibrary() != null) {
                String version = jbiConfig.getSharedLibrary().getVersion();
                if (version == null || version.trim().isEmpty()) {
                    throw new ArtifactSlVersionMissingInJBIDescriptorException(artifactUrl);
                }
                return new SharedLibrary(getName(jbiConfig.getSharedLibrary().getIdentification(), artifactUrl),
                        version);
            } else if (jbiConfig.getServiceAssembly() != null) {
                final List<ServiceUnit> sus = new ArrayList<>();
                for (org.ow2.petals.jbi.descriptor.original.generated.ServiceUnit su : jbiConfig.getServiceAssembly()
                        .getServiceUnit()) {
                    sus.add(new ServiceUnit(su.getIdentification().getName(), su.getTarget().getComponentName()));
                }
                return new ServiceAssembly(getName(jbiConfig.getServiceAssembly().getIdentification(), artifactUrl),
                        sus);
            } else {
                JBIDescriptorExtensionBuilder builder = JBIDescriptorExtensionBuilder.getInstance();
                final org.ow2.petals.jbi.descriptor.extension.generated.Identification identification;
                try {
                    identification = builder.getDeployableServiceUnitIndentification(jbiConfig);
                } catch (NotDeployableServiceUnitException | NoIdentificationDeployableServiceUnitException e) {
                    throw new ArtifactNameMissingInJBIDescriptorException(artifactUrl);
                } catch (SeveralIdentificationDeployableServiceUnitException e) {
                    throw new SeveralServicesException(artifactUrl);
                } 
                final String targetComponent;
                try {
                    targetComponent = builder.getDeployableServiceUnitTargetComponent(jbiConfig);
                } catch (NoComponentNameDeployableServiceUnitException e) {
                    throw new TargetComponentMissingException(artifactUrl);
                }
                if (targetComponent == null || targetComponent.trim().isEmpty()) {
                    throw new TargetComponentMissingException(artifactUrl);
                }
                checkName(identification.getName(), artifactUrl);

                return new ServiceUnit(identification.getName(), targetComponent);
            }
        } catch (final IOException e) {
            throw new ArtifactException(e);
        } catch (JBIDescriptorException e) {
            final String errorMessage = String.format("%s: not a valid JBI archive", artifactUrl.toString());
            throw new InvalidZipArchiveException(errorMessage, artifactUrl, e);
        }
    }

    private static String getName(Identification identification, URL artifactUrl)
            throws ArtifactNameMissingInJBIDescriptorException {
        checkName(identification.getName(), artifactUrl);
        return identification.getName();
    }

    private static void checkName(String name, URL artifactUrl) throws ArtifactNameMissingInJBIDescriptorException {
        if (name == null || name.trim().isEmpty()) {
            throw new ArtifactNameMissingInJBIDescriptorException(artifactUrl);
        }
    }

    private static final File downloadPropertiesFileToTempZipFile(final URL propertiesUrl)
            throws HttpRemoteArtifactException, IOException {

        final InputStream propertiesStream;
        if ("http".equalsIgnoreCase(propertiesUrl.getProtocol())) {
            final HttpURLConnection httpUrlConnection = (HttpURLConnection) propertiesUrl.openConnection();
            final int responseCode = httpUrlConnection.getResponseCode();
            if (responseCode != HttpURLConnection.HTTP_OK) {
                throw new HttpCodeNotOkException(propertiesUrl, responseCode);
            } else {
                // We check that the resource is a binary file
                final String contentType = httpUrlConnection.getContentType();
                if (contentType != null) {
                    if (!contentType.startsWith(MimeTypeConstants.TEXT_PLAIN)
                            && !contentType.startsWith(MimeTypeConstants.TEXT_X_JAVA_PROPERTIES)) {
                        throw new InvalidContentTypeException(propertiesUrl, contentType);
                    }
                } else {
                    throw new MissingContentTypeException(propertiesUrl);
                }
            }

            propertiesStream = httpUrlConnection.getInputStream();
        } else {
            propertiesStream = propertiesUrl.openStream();
        }

        try {
            final File result = File.createTempFile("jbi", ".properties");
            result.deleteOnExit();
            final OutputStream output = new FileOutputStream(result);
            try {
                IOUtils.copy(propertiesStream, output);
                return result;
            } finally {
                output.close();
            }
        } finally {
            propertiesStream.close();
        }
    }

    private static final File downloadArtifactToTempZipFile(final URL artifactUrl) throws HttpRemoteArtifactException,
            IOException {

        final InputStream artifactStream;
        if ("http".equalsIgnoreCase(artifactUrl.getProtocol())) {
            final HttpURLConnection httpUrlConnection = (HttpURLConnection) artifactUrl.openConnection();
            final int responseCode = httpUrlConnection.getResponseCode();
            if (responseCode != HttpURLConnection.HTTP_OK) {
                throw new HttpCodeNotOkException(artifactUrl, responseCode);
            } else {
                // We check that the resource is a binary file
                final String contentType = httpUrlConnection.getContentType();
                if (contentType != null) {
                    if (!contentType.startsWith(MimeTypeConstants.APPLICATION_X_ZIP)
                            && !contentType.startsWith(MimeTypeConstants.APPLICATION_X_ZIP_COMPRESSED)
                            && !contentType.startsWith(MimeTypeConstants.APPLICATION_ZIP)
                            && !contentType.startsWith(MimeTypeConstants.APPLICATION_ZIP_COMPRESSED)) {
                        throw new InvalidContentTypeException(artifactUrl, contentType);
                    }
                } else {
                    throw new MissingContentTypeException(artifactUrl);
                }
            }

            artifactStream = httpUrlConnection.getInputStream();
        } else {
            artifactStream = artifactUrl.openStream();
        }

        try {
            final File result = File.createTempFile("jbi", ".zip");
            result.deleteOnExit();
            final OutputStream output = new FileOutputStream(result);
            try {
                IOUtils.copy(artifactStream, output);
                return result;
            } finally {
                output.close();
            }
        } finally {
            artifactStream.close();
        }
    }

    private static Set<SharedLibrary> getSLs(
            final org.ow2.petals.jbi.descriptor.original.generated.Component componentConfig) {
        Set<SharedLibrary> sls = new HashSet<>();
        for (org.ow2.petals.jbi.descriptor.original.generated.Component.SharedLibrary sl : componentConfig
                .getSharedLibraryList()) {
            sls.add(new SharedLibrary(sl.getContent(), sl.getVersion()));
        }
        return sls;
    }
}
