/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api.artifact;

import java.util.HashSet;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.admin.api.artifact.exception.ConfigurationParamComputeException;

/**
 * This class represents a remote JBI Component.
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class Component extends Artifact implements ArtifactState {

    public enum ComponentType {
        BC, SE
    }

    private final ComponentType componentType;

    private final ConfigurationPropertiesCallback configurationPropertiesCallback;

    private Properties parameters;

    private State state;

    private Set<SharedLibrary> sharedLibraries;

    public Component(final String name, final ComponentType componentType) {
        this(name, componentType, new HashSet<SharedLibrary>());
    }

    public Component(final String name, final ComponentType componentType, State state) {
        this(name, componentType, state, new HashSet<SharedLibrary>());
    }

    public Component(final String name, final ComponentType type,
            final ConfigurationPropertiesCallback configurationPropertiesCallback) {
        this(name, type, new HashSet<SharedLibrary>(), configurationPropertiesCallback);
    }

    public Component(final String name, final ComponentType type, Set<SharedLibrary> sharedLibraries,
            final ConfigurationPropertiesCallback configurationPropertiesCallback) {
        this(name, type, State.UNKNOWN, sharedLibraries, null, configurationPropertiesCallback);
    }

    public Component(final String name, final ComponentType componentType, final State state,
            final ConfigurationPropertiesCallback configurationPropertiesCallback) {
        this(name, componentType, state, new HashSet<SharedLibrary>(), null, configurationPropertiesCallback);
    }

    public Component(final String name, final ComponentType componentType, final Properties parameters) {
        this(name, componentType, new HashSet<SharedLibrary>(), parameters);
    }

    public Component(final String name, final ComponentType componentType, Set<SharedLibrary> sharedLibraries) {
        this(name, componentType, sharedLibraries, new Properties());
    }

    public Component(final String name, final ComponentType componentType, Set<SharedLibrary> sharedLibraries,
            final Properties parameters) {
        this(name, componentType, State.UNKNOWN, sharedLibraries, parameters, null);
    }

    public Component(final String name, final ComponentType componentType, final State state,
            final Properties parameters) {
        this(name, componentType, state, new HashSet<SharedLibrary>(), parameters, null);
    }

    public Component(final String name, final ComponentType componentType, State state,
            Set<SharedLibrary> sharedLibraries) {
        this(name, componentType, state, sharedLibraries, new Properties(), null);
    }

    private Component(final String name, final ComponentType componentType, final State state,
            Set<SharedLibrary> sharedLibraries, final Properties parameters,
            final ConfigurationPropertiesCallback configurationPropertiesCallback) {
        super(name, componentType.toString());
        assert parameters == null ^ configurationPropertiesCallback == null;
        this.componentType = componentType;
        this.parameters = parameters == null ? new Properties() : parameters;
        this.state = state;
        this.sharedLibraries = sharedLibraries == null ? new HashSet<SharedLibrary>() : sharedLibraries;
        this.configurationPropertiesCallback = configurationPropertiesCallback;
    }

    /**
     * Configure and set configuration parameters from old ones
     * 
     * @param oldConfigParam
     *            The old configuration parameters
     * @throws ConfigurationParamComputeException
     */
    public final Properties configure(final Map<String, String> oldConfigParam)
            throws ConfigurationParamComputeException {
        if (this.configurationPropertiesCallback != null) {
            return this.configurationPropertiesCallback.compute(oldConfigParam);
        } else {
            Properties properties = new Properties();
            properties.putAll(this.parameters);
            return properties;
        }
    }

    /**
     * Get the component type
     * 
     * @return the component type
     */
    public final ComponentType getComponentType() {
        return this.componentType;
    }

    /**
     * Get the parameters
     * 
     * @return the parameters
     */
    public Properties getParameters() {
        return this.parameters;
    }

    @Override
    public State getState() {
        return this.state;
    }

    @Override
    public void setState(State state) {
        this.state = state;
    }

    public Set<SharedLibrary> getSharedLibraries() {
        return this.sharedLibraries;
    }

    @Override
    public Component copy() {
        Set<SharedLibrary> sls = new HashSet<>();
        for (SharedLibrary sl : this.sharedLibraries) {
            sls.add(sl.copy());
        }
        Properties properties = new Properties();
        properties.putAll(this.parameters);
        return new Component(this.getName(), this.componentType, this.state, sls, properties,
                this.configurationPropertiesCallback);
    }

}
