/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api.artifact.lifecycle;

import java.net.URL;

import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactUnsupportedOperationException;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public interface ComponentLifecycle extends ArtifactLifecycle {

    public Component getComponent();

    /**
     * <p>
     * Install/deploy the artifact located at the given {@link URL}
     * </p>
     * <p>
     * <b>Note</b>: if the given URL is not a local URL, it is transformed according to {@link} before the real
     * deployment. A local URL is an URL with a scheme 'file', or 'http' on the local host.
     * 
     * @param artifactUrl
     *            The {@link URL} of the artifact to deploy or install
     * @param skipInstall
     *            Skip the installation of the component if it is <code>true</code>
     * @throws ArtifactDeployedException
     *             The artifact is already deployed.
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact deployment.
     */
    public void deploy(final URL artifactUrl, final boolean skipInstall)
            throws ArtifactDeployedException, ArtifactAdministrationException;

    /**
     * Install a previously deployed component
     *
     * @throws ArtifactNotFoundException
     *             The current component is not found on the Petals container side
     * @throws ArtifactDeployedException
     *             The artifact is already deployed.
     * @throws ArtifactAdministrationException
     *             An unexpected error occurs
     */
    public void install() throws ArtifactDeployedException, ArtifactNotFoundException, ArtifactAdministrationException;

    /**
     * Uninstall a previously installed component
     *
     * @throws ArtifactNotFoundException
     *             The current component is not found on the Petals container side
     * @throws ArtifactStartedException
     *             The artifact is already started
     * @throws ArtifactNotDeployedException
     *             The artifact is not deployed
     * @throws ArtifactAdministrationException
     *             An unexpected error occurs
     */
    public void uninstall() throws ArtifactNotDeployedException, ArtifactStartedException, ArtifactNotFoundException,
            ArtifactAdministrationException;

    /**
     * Reload placeholders configuration of the current component
     * 
     * @throws ArtifactNotFoundException
     *             The current component is not found on the Petals container side
     * @throws ArtifactUnsupportedOperationException
     *             This operation is not supported by the current component
     * @throws ArtifactAdministrationException
     *             An unexpected error occurs
     */
    public void reloadConfiguration()
            throws ArtifactNotFoundException, ArtifactUnsupportedOperationException, ArtifactAdministrationException;

    /**
     * Set the parameters
     * 
     * @throws ArtifactNotFoundException
     *             The current component is not found on the Petals container side
     * @throws ArtifactAdministrationException
     *             An unexpected error occurs
     */
    public void setParameters() throws ArtifactNotFoundException, ArtifactAdministrationException;

    /**
     * Update the state and the parameters of the component
     */
    public void updateStateAndParameters() throws ArtifactNotFoundException, ArtifactAdministrationException;
}
