/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.topology;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;

/**
 * 
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class Container {

    public enum PortType {
        
        JMX("JMX port"),
        TCP_TRANSPORT("TCP port");
        
        private String displayValue;
        
        PortType(final String displayValue) {
            this.displayValue = displayValue;
        }

        @Override
        public String toString() {
            return this.displayValue;
        }
    }

    public enum State {

        REACHABLE("Reachable"), UNREACHABLE("Unreachable"), UNKNOWN("Unknown");

        private String displayValue;

        State(final String displayValue) {
            this.displayValue = displayValue;
        }
    
        @Override
        public String toString(){
            return this.displayValue;
        }
    }
    
    private final String containerName;

    private final String host;
    
    private final Map<PortType, Integer> ports;
    
    private final String jmxUsername;

    private final String jmxPassword;

    private final State state;

    /**
     * The container local configuration (ie. server.properties)
     */
    private final Properties localConfiguration = new Properties();

    private final List<SharedLibrary> sharedLibraries = new ArrayList<>();

    private final List<Component> components = new ArrayList<>();

    private final List<ServiceAssembly> serviceAssemblies = new ArrayList<>();

    public Container(final String containerName, final String host, final Map<PortType, Integer> ports,
            final String jmxUsername, final String jmxPassword, final State state) {
        assert  containerName != null;
        assert  host != null;
        assert  ports != null;
        
        this.containerName = containerName;
        this.host = host;
        this.ports = ports;
        this.jmxPassword = jmxPassword;
        this.jmxUsername = jmxUsername;
        this.state = state;
    }

    public String getContainerName() {
        return this.containerName;
    }
    
    public String getHost() {
        return this.host;
    }
    
    public Map<PortType, Integer> getPorts() {
        return this.ports;
    }

    public String getJmxUsername() {
        return this.jmxUsername;
    }

    public String getJmxPassword() {
        return this.jmxPassword;
    }

    public State getState() {
        return this.state;
    }

    /**
     * @return The share library list deployed on the current container. Not null.
     */
    public List<SharedLibrary> getSharedLibraries() {
        return Collections.unmodifiableList(this.sharedLibraries);
    }

    /**
     * Add a shared library as deployed on the current container
     * 
     * @param sharedLibrary
     *            The shared library
     */
    public void addSharedLibrary(final SharedLibrary sharedLibrary) {
        this.sharedLibraries.add(sharedLibrary);
    }

    /**
     * Remove a shared library from the current container
     * 
     * @param sharedLibrary
     *            The shared library
     */
    public void removeSharedLibrary(final SharedLibrary sharedLibrary) {
        this.sharedLibraries.remove(sharedLibrary);
    }

    /**
     * @return The component list deployed on the current container. Not null.
     */
    public List<Component> getComponents() {
        return Collections.unmodifiableList(this.components);
    }

    /**
     * Add a component as deployed on the current container
     * 
     * @param component
     *            The component
     */
    public void addComponent(final Component component) {
        this.components.add(component);
    }

    /**
     * Remove a component from the current container
     * 
     * @param component
     *            The component
     */
    public void removeComponent(final Component component) {
        this.components.remove(component);
    }

    /**
     * @return The service assembly list deployed on the current container. Not null.
     */
    public List<ServiceAssembly> getServiceAssemblies() {
        return Collections.unmodifiableList(this.serviceAssemblies);
    }

    /**
     * Add a service assembly as deployed on the current container
     * 
     * @param serviceAssembly
     *            The service assembly
     */
    public void addServiceAssembly(final ServiceAssembly serviceAssembly) {
        this.serviceAssemblies.add(serviceAssembly);
    }

    /**
     * Remove a service assembly from the current container
     * 
     * @param serviceAssembly
     *            The service assembly
     */
    public void removeServiceAssembly(final ServiceAssembly serviceAssembly) {
        this.serviceAssemblies.remove(serviceAssembly);
    }

    /**
     * Add a property to the container local configuration (ie. server.properties).
     */
    public void addProperty(final String propertyName, final String value) {
        this.localConfiguration.setProperty(propertyName, value);
    }

    public Properties getLocalConfiguration() {
        return this.localConfiguration;
    }

    @Override
    public String toString() {
        return "Container[" + this.containerName + "," + this.host + "," + this.ports + "]";
    }

    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((containerName == null) ? 0 : containerName.hashCode());
        result = prime * result + ((host == null) ? 0 : host.hashCode());
        result = prime * result + ((ports == null) ? 0 : ports.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        Container other = (Container) obj;
        if (containerName == null) {
            if (other.containerName != null)
                return false;
        } else if (!containerName.equals(other.containerName))
            return false;
        if (host == null) {
            if (other.host != null)
                return false;
        } else if (!host.equals(other.host))
            return false;
        if (ports == null) {
            if (other.ports != null)
                return false;
        } else if (!ports.equals(other.ports))
            return false;
        return true;
    }

}