/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api;

import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.zip.ZipException;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.admin.api.artifact.AbstractArtifactTest;
import org.ow2.petals.admin.api.artifact.ArtifactUtilsTest;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.Component.ComponentType;
import org.ow2.petals.admin.api.artifact.MimeTypeConstants;
import org.ow2.petals.admin.api.artifact.exception.HttpCodeNotOkException;
import org.ow2.petals.admin.api.artifact.exception.InvalidContentTypeException;
import org.ow2.petals.admin.api.artifact.exception.InvalidZipArchiveException;
import org.ow2.petals.admin.api.artifact.exception.MissingContentTypeException;
import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ComponentRequiredException;
import org.ow2.petals.admin.api.exception.UnsupportedArtifactTypeException;
import org.ow2.petals.junit.rules.webserver.WebServer;
import org.ow2.petals.junit.rules.webserver.servlet.AbstractHttpServlet;
import org.ow2.petals.junit.rules.webserver.servlet.FileServlet;
import org.ow2.petals.junit.rules.webserver.servlet.TextServlet;

/**
 * Unit tests of {@link ArtifactAdministration}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ArtifactAdministrationTest extends AbstractArtifactTest {
    
    private final String COMPONENT_NAME_BC_SUPPORTING_RELOADING = "bc-supporting-reloading";

    private final String COMPONENT_NAME_SE_SUPPORTING_RELOADING = "se-supporting-reloading";

    private ArtifactAdministration artifactAdministration = new PetalsAdministrationFactoryMock()
            .registerArtifact(new Component(COMPONENT_NAME_BC_SUPPORTING_RELOADING, ComponentType.BC))
            .registerArtifact(new Component(COMPONENT_NAME_SE_SUPPORTING_RELOADING, ComponentType.SE))
            .newPetalsAdministrationAPI().newArtifactAdministration();

    @Rule
    public final WebServer webServer = new WebServer();

    /**
     * <p>
     * Try to deploy a binding-component available on a local filesystem.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_BindingComponent() throws ZipException, ArtifactAdministrationException,
            IOException, URISyntaxException {

        this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.BC_JBI), false);
    }

    /**
     * <p>
     * Try to deploy a service-engine available on a local filesystem.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_ServiceEngine() throws ZipException, ArtifactAdministrationException,
            IOException, URISyntaxException {

        this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.SE_JBI), false);

    }

    /**
     * <p>
     * Try to deploy a service-assembly available on a local filesystem.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_ServiceAssembly() throws ZipException, ArtifactAdministrationException,
            IOException, URISyntaxException {

        this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.SA_JBI), false);
    }

    /**
     * <p>
     * Try to deploy a shared-library available on a local filesystem.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_SharedLibrary() throws ZipException, ArtifactAdministrationException,
            IOException, URISyntaxException {

        this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.SL_JBI), false);
    }

    /**
     * <p>
     * Try to deploy a deployable service unit available on a local filesystem.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_DeployableServiceUnit() throws ZipException, ArtifactAdministrationException,
            IOException, URISyntaxException {

        this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.DEPLOYABLE_SU_JBI),
                false);
    }

    /**
     * <p>
     * Try to deploy a standard service unit.
     * </p>
     * <p>
     * Expected result: an error occurs (missing target component identifier) because the service unit is not a
     * deployable service unit
     * </p>
     */
    @Test(expected = InvalidZipArchiveException.class)
    public void deployAndStartArtifact_StandardServiceUnit() throws Throwable {

        try {
            this.artifactAdministration.deployAndStartArtifact(
                    this.createArtifactURL(ArtifactUtilsTest.STANDARD_SU_JBI), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a deployable service unit available on a HTTP web server.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_DeployableServiceUnitOnHTTP() throws ZipException, ArtifactAdministrationException,
            IOException, URISyntaxException {
        
        final File localArtifact = new File(this.createArtifactURL(ArtifactUtilsTest.DEPLOYABLE_SU_JBI).toURI());
        final AbstractHttpServlet fileServlet = new FileServlet(localArtifact, MimeTypeConstants.APPLICATION_ZIP);

        this.webServer.addServlet(fileServlet, fileServlet.getPath());

        this.artifactAdministration.deployAndStartArtifact(
                new URL(this.webServer.getHttpBaseUrl() + fileServlet.getPath()), false);
    }

    /**
     * <p>
     * Try to deploy a deployable service unit available on a HTTP web server, where:
     * <ul>
     * <li>no content-type is returned in the HTTP response.</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the missing content-type
     * </p>
     */
    @Test(expected = MissingContentTypeException.class)
    public void deployAndStartArtifact_DeployableServiceUnitOnHTTPMissingContentType() throws Throwable {

        final File localArtifact = new File(this.createArtifactURL(ArtifactUtilsTest.DEPLOYABLE_SU_JBI).toURI());
        final AbstractHttpServlet fileServlet = new FileServlet(localArtifact, null);

        this.webServer.addServlet(fileServlet, fileServlet.getPath());

        try {
            this.artifactAdministration.deployAndStartArtifact(
                    new URL(this.webServer.getHttpBaseUrl() + fileServlet.getPath()), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a deployable service unit available on a HTTP web server, where:
     * <ul>
     * <li>the returned content type is not associated to ZIP file.</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about invalid content type
     * </p>
     */
    @Test(expected = InvalidContentTypeException.class)
    public void deployAndStartArtifact_DeployableServiceUnitOnHTTPWithInvalidContentType() throws Throwable {

        final AbstractHttpServlet textServlet = new TextServlet();

        this.webServer.addServlet(textServlet, textServlet.getPath());

        try {
            this.artifactAdministration.deployAndStartArtifact(
                    new URL(this.webServer.getHttpBaseUrl() + textServlet.getPath()), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a deployable service unit available on a HTTP web server, where:
     * <ul>
     * <li>the artifact URL does not exist on the HTTP server side (ie. Error 404).</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the inexisting remote artifact
     * </p>
     */
    @Test(expected = HttpCodeNotOkException.class)
    public void deployAndStartArtifact_DeployableServiceUnitOnHTTPWith404() throws Throwable {

        try {
            this.artifactAdministration.deployAndStartArtifact(new URL(this.webServer.getHttpBaseUrl()
                    + "/unexisting-artifact.zip"), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a binding component available on the local file system, where:
     * <ul>
     * <li>a remote configuration file is provided as HTTP resource,</li>
     * <li>the remote configuration file does not exist on the HTTP server side (ie. Error 404).</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the inexisting configuration file
     * </p>
     */
    @Test(expected = HttpCodeNotOkException.class)
    public void deployAndStartArtifact_LocalComponentWithRemoteConfFileWithHTTP404() throws Throwable {

        try {
            this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.BC_JBI),
                    new URL(this.webServer.getHttpBaseUrl() + "/unexisting-cfg-file.properties"), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a binding component available on the local file system, where:
     * <ul>
     * <li>a remote configuration file is provided as HTTP resource,</li>
     * <li>the returned content type is not associated to a properties file.</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the invalid content-type of the configuration file
     * </p>
     */
    @Test(expected = InvalidContentTypeException.class)
    public void deployAndStartArtifact_LocalComponentWithRemoteConfFileWithInvalidContentType() throws Throwable {

        final File cfgFile = tempFolder.newFile();
        final AbstractHttpServlet fileServlet = new FileServlet(cfgFile, MimeTypeConstants.APPLICATION_ZIP);

        this.webServer.addServlet(fileServlet, fileServlet.getPath());

        try {
            this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.BC_JBI),
                    new URL(this.webServer.getHttpBaseUrl() + fileServlet.getPath()), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a binding component available on the local file system, where:
     * <ul>
     * <li>a remote configuration file is provided as HTTP resource,</li>
     * <li>no content-type is returned in the HTTP response.</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: an error occurs about the missing content-type of the configuration file
     * </p>
     */
    @Test(expected = MissingContentTypeException.class)
    public void deployAndStartArtifact_LocalComponentWithRemoteConfFileWithMissingContentType() throws Throwable {

        final File cfgFile = tempFolder.newFile();
        final AbstractHttpServlet fileServlet = new FileServlet(cfgFile, null);

        this.webServer.addServlet(fileServlet, fileServlet.getPath());

        try {
            this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.BC_JBI),
                    new URL(this.webServer.getHttpBaseUrl() + fileServlet.getPath()), false);
        } catch (final ArtifactAdministrationException e) {
            throw e.getCause();
        }
    }

    /**
     * <p>
     * Try to deploy a binding component available on the local file system, where:
     * <ul>
     * <li>a remote configuration file is provided as HTTP resource.</li>
     * </ul>
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void deployAndStartArtifact_LocalComponentWithRemoteConfFile() throws ZipException,
            ArtifactAdministrationException, IOException, URISyntaxException {

        final File cfgFile = tempFolder.newFile();
        final AbstractHttpServlet fileServlet = new FileServlet(cfgFile, MimeTypeConstants.TEXT_PLAIN);

        this.webServer.addServlet(fileServlet, fileServlet.getPath());

        this.artifactAdministration.deployAndStartArtifact(this.createArtifactURL(ArtifactUtilsTest.BC_JBI), new URL(
                this.webServer.getHttpBaseUrl() + fileServlet.getPath()), false);
    }

    /**
     * <p>
     * Try to reload configuration of a binding component available on the local file system through an URL.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void reloadConfiguration_LocalBindingComponentByURL() throws ArtifactNotFoundException,
            ComponentRequiredException, ZipException, ArtifactAdministrationException, IOException, URISyntaxException {

        this.artifactAdministration.reloadConfiguration(this.createArtifactURL(ArtifactUtilsTest.BC_JBI));
    }

    /**
     * <p>
     * Try to reload configuration of a service assembly available on the local file system through an URL.
     * </p>
     * <p>
     * Expected result: an error occurs about only component artifact is accepted
     * </p>
     */
    @Test(expected = ComponentRequiredException.class)
    public void reloadConfiguration_LocalServiceAssemblyByURL() throws ArtifactNotFoundException,
            ComponentRequiredException, ZipException, ArtifactAdministrationException, IOException, URISyntaxException {

        this.artifactAdministration.reloadConfiguration(this.createArtifactURL(ArtifactUtilsTest.SA_JBI));
    }

    /**
     * <p>
     * Try to reload configuration of a binding component previously installed supporting configuration reloading.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void reloadConfiguration_BindingComponentSupportingReloading() throws UnsupportedArtifactTypeException,
            ArtifactNotFoundException, ArtifactAdministrationException {

        this.artifactAdministration.reloadConfiguration(COMPONENT_NAME_BC_SUPPORTING_RELOADING);
    }

    /**
     * <p>
     * Try to reload configuration of a service engine previously installed supporting configuration reloading.
     * </p>
     * <p>
     * Expected result: no error occurs
     * </p>
     */
    @Test
    public void reloadConfiguration_ServiceEngineSupportingReloading() throws UnsupportedArtifactTypeException,
            ArtifactNotFoundException, ArtifactAdministrationException {

        this.artifactAdministration.reloadConfiguration(COMPONENT_NAME_SE_SUPPORTING_RELOADING);
    }

    /**
     * <p>
     * Try to reload configuration of a service engine not found.
     * </p>
     * <p>
     * Expected result: an error occurs about the not found component
     * </p>
     */
    @Test(expected = ArtifactNotFoundException.class)
    public void reloadConfiguration_ServiceEngineNotFound() throws UnsupportedArtifactTypeException,
            ArtifactNotFoundException, ArtifactAdministrationException {

        this.artifactAdministration.reloadConfiguration("se-not-found");
    }

}
